package org.nakedobjects.nof.reflect.i18n;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;
import org.nakedobjects.nof.reflect.peer.MemberIdentifier;


public class ResourceBasedI18nManager implements I18nManager {
    private static final Logger LOG = Logger.getLogger(ResourceBasedI18nManager.class);
    private static final String BASE_FILE_NAME = "i18n";
    private static final String PARAMETER = "parameter";
    private static final String DESCRIPTION = "description";
    private static final String PROPERTY = "property";
    private static final String NAME = "name";
    private static final String HELP = "help";
    private static final String ACTION = "action";

    private ResourceBundle bundle;

    public String getDescription(final MemberIdentifier identifier) {
        return text(identifier, DESCRIPTION);
    }

    public String getHelp(final MemberIdentifier identifier) {
        return text(identifier, HELP);
    }

    public String getName(final MemberIdentifier identifier) {
        return text(identifier, NAME);
    }

    // TODO allow description and help to be found for parameters
    public String[] getParameterNames(final MemberIdentifier identifier) {
        if (bundle == null) {
            return null;
        } else {
            String[] array = new String[identifier.getParameters().length];
            for (int i = 0; i < array.length; i++) {
                String key = identifier.getClassName() + "." + ACTION + "." + identifier.getName() + "." + PARAMETER + (i + 1) + "." + NAME;
                try {
                    array[i] = bundle.getString(key);
                } catch (MissingResourceException e) {
                    array[i] = null;
                }
            }
            return array;
        }
    }

    public void init() {
        try {
            bundle = ResourceBundle.getBundle(BASE_FILE_NAME);
        } catch (MissingResourceException e) {
            LOG.warn("Missing resource bundle: " + e.getMessage());
        }
            
    }

    public void shutdown() {}

    private String text(final MemberIdentifier identifier, final String type) {
        if (bundle == null) {
            return null;
        } else {
            String form = identifier.isField() ? PROPERTY : ACTION;
            String key = identifier.getClassName() + "." + form + "." + identifier.getName() + "." + type;
            try {
                return bundle.getString(key);
            } catch (MissingResourceException e) {
                return null;
            }
        }
    }

}

// Copyright (c) Naked Objects Group Ltd.
