package org.nakedobjects.nof.reflect.i18n;

import org.nakedobjects.nof.reflect.peer.AbstractValuePeer;
import org.nakedobjects.nof.reflect.peer.ValuePeer;


/**
 * One to one authorisation decorator adds checks via the <code>AuthorisationManager</code> to
 * <code>isAuthorised</code> and <code>isAvailable</code> calls.
 */
public class ValueI18n extends AbstractValuePeer {
    private final I18nManager i18nManager;

    public ValueI18n(final ValuePeer local, final I18nManager i18nManager) {
        super(local);
        this.i18nManager = i18nManager;
    }

    public String getDescription() {
        String description = i18nManager.getDescription(getIdentifier());
        if (description == null) {
            return super.getDescription();
        } else {
            return description;
        }
    }

    public String getName() {
        String name = i18nManager.getName(getIdentifier());
        if (name == null) {
            return super.getName();
        } else {
            return name;
        }
    }

    public String getHelp() {
        String help = i18nManager.getHelp(getIdentifier());
        if (help == null) {
            return super.getHelp();
        } else {
            return help;
        }
    }
}
// Copyright (c) Naked Objects Group Ltd.
