package org.nakedobjects.nof.reflect.perspective;

import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.context.Perspective;
import org.nakedobjects.nof.core.reflect.AbstractConsent;
import org.nakedobjects.nof.core.reflect.Allow;
import org.nakedobjects.nof.reflect.peer.OneToManyPeer;

import java.util.List;


abstract class PerspectiveCollectionPeer extends PerspectiveFieldPeer implements OneToManyPeer {
    private NakedObjectSpecification type = NakedObjectsContext.getReflector().loadSpecification(Object.class);

    public void addAssociation(NakedObject inObject, NakedObject element) {
        if (isAddValid(inObject, element).isAllowed()) {
            List list = getList(((Perspective) inObject.getObject()));
            list.add(element.getObject());
            objectChanged(inObject);
        }
    }

    private void objectChanged(final NakedObject inObject) {
        NakedObjectsContext.getObjectPersistor().objectChanged(inObject);
    }
    
    public NakedCollection getAssociations(NakedObject inObject) {
        List list = getList(((Perspective) inObject.getObject()));
        return NakedObjectsContext.getObjectLoader().getAdapterForElseCreateAdapterForCollection(inObject, getName(), type, list);
    }

    protected abstract List getList(Perspective perspective);

    public NakedObjectSpecification getSpecification() {
        return type;
    }

    public void initAssociation(NakedObject inObject, NakedObject associate) {
        List list = getList(((Perspective) inObject.getObject()));
        list.add(associate.getObject());
    }

    public void initOneToManyAssociation(NakedObject inObject, NakedObject[] instances) {
        List list = getList(((Perspective) inObject.getObject()));
        list.clear();
        for (int i = 0; i < instances.length; i++) {
            list.add(instances[i].getObject());
        }
    }

    public Consent isAddValid(NakedObject inObject, NakedObject element) {
        List list = getList(((Perspective) inObject.getObject()));
        boolean alreadyContainsElement = list.contains(element.getObject());
        return AbstractConsent.create(!alreadyContainsElement, "", "Already contains this object");
    }

    public Consent isRemoveValid(NakedObject container, NakedObject element) {
        return Allow.DEFAULT;
    }

    public void removeAllAssociations(NakedObject inObject) {
        List list = getList(((Perspective) inObject.getObject()));
        list.clear();
    }

    public void removeAssociation(NakedObject inObject, NakedObject element) {
        if (isRemoveValid(inObject, element).isAllowed()) {
            List list = getList(((Perspective) inObject.getObject()));
            list.remove(element.getObject());
            objectChanged(inObject);
        }
    }
        
}

// Copyright (c) Naked Objects Group Ltd.
