package org.nakedobjects.nof.reflect.spec;

import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.reflect.NakedObjectActionParameter;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.reflect.Allow;
import org.nakedobjects.nof.reflect.peer.ActionParamPeer;

public abstract class AbstractActionParameter implements NakedObjectActionParameter {


    protected AbstractActionParameter(
            int number, 
            NakedObjectAction nakedObjectAction, 
            final ActionParamPeer peer, 
            NakedObjectSpecification type, 
            String name, 
            String description, 
            boolean optional) {
        this.number = number;
        this.action = nakedObjectAction;
        this.type = type;
        this.name = name;
        this.description = description;
        this.optional = optional;
        this.peer = peer;
    }

    private ActionParamPeer peer;


    /**
     * Subclasses should override precisely one of 
     * {@link #isObject()}, {@link #isCollection()} and {@link #isValue()}.
     */
    public boolean isObject() {
        return false;
    }

    /**
     * Subclasses should override precisely one of 
     * {@link #isObject()}, {@link #isCollection()} and {@link #isValue()}.
     */
    public boolean isValue() {
        return false;
    }

    /**
     * Subclasses should override precisely one of 
     * {@link #isObject()}, {@link #isCollection()} and {@link #isValue()}.
     */
    public boolean isCollection() {
        return false;
    }

    private final int number;
    public int getNumber() {
        return number;
    }
    
    private NakedObjectAction action;
    public NakedObjectAction getAction() {
        return action;
    }

    private NakedObjectSpecification type;
    public NakedObjectSpecification getSpecification() {
        return type;
    }

    private final String name;
    public String getName() {
        return name;
    }

    private final String description;
    public String getDescription() {
        return description;
    }

    private final boolean optional;
    public boolean isOptional() {
        return optional;
    }

    public Consent isUsable() {
        return Allow.DEFAULT;
    }

    

    public Object getExtension(Class cls) {
        return peer != null? peer.getExtension(cls): null;
    }

    public Class[] getExtensions() {
        return peer != null? peer.getExtensions(): new Class[]{};
    }


}
