package org.nakedobjects.nof.reflect.spec;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedReference;
import org.nakedobjects.noa.adapter.Persistable;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.reflect.NakedObjectFeature;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.reflect.NakedObjectAction.Type;
import org.nakedobjects.noa.reflect.NakedObjectField.Filter;
import org.nakedobjects.noa.spec.Features;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.reflect.Allow;
import org.nakedobjects.nof.core.util.ToString;
import org.nakedobjects.nof.core.util.UnexpectedCallException;


// TODO work through all subclasses and remove duplicated overridden methods - this 
// hierarchy is badly structured move common, and default, functionality to this class from subclasses.
public abstract class AbstractSpecification implements NakedObjectSpecification {
    protected String className;
    private Map extensionsByClass = new HashMap();
    protected NakedObjectField[] fields;
    protected NakedObjectAction[] objectActions;
    protected NakedObjectSpecification superClassSpecification;

    /**
     * Not part of the API.
     * 
     * <p>
     * However subclasses such as <tt>JavaSpecification</tt> promote this to an (internal) API through
     * implementing <tt>ExtensionHolderMutable</tt>.
     * 
     * @param extension
     */
    protected void addExtension(Object extension) {
        extensionsByClass.put(extension.getClass(), extension);
    }

    public void addSubclass(NakedObjectSpecification specification) {}

    public void clearDirty(NakedObject object) {}

    /*
     * Add all relevant methods with a set for each service
     */
    private NakedObjectAction[] findServiceMethodsWithParameter(final NakedObjectAction.Type type) {
        Object[] services = NakedObjectsContext.getObjectLoader().getServices();
        List serviceActionSets = new ArrayList();
        for (int i = 0; i < services.length; i++) {
            NakedObject serviceAdapter = NakedObjectsContext.getObjectLoader().getAdapterFor(services[i]);
            NakedObjectSpecification specification = serviceAdapter.getSpecification();
            if (specification == this) {
                continue;
            }
            NakedObjectAction[] serviceActions = specification.getObjectActions(type);
            List matchingServiceActions = new ArrayList();
            for (int j = 0; j < serviceActions.length; j++) {
                NakedObjectSpecification[] params = serviceActions[j].getParameterTypes();
                for (int k = 0; k < params.length; k++) {
                    if (isOfType(params[k])) {
                        matchingServiceActions.add(serviceActions[j]);
                        break;
                    }
                }
            }
            if (matchingServiceActions.size() > 0) {
                NakedObjectAction[] asArray = (NakedObjectAction[]) matchingServiceActions
                        .toArray(new NakedObjectAction[matchingServiceActions.size()]);
                ActionSet actionSet = new ActionSet("id", serviceAdapter.titleString(), asArray);
                serviceActionSets.add(actionSet);
            }

        }
        return (NakedObjectAction[]) serviceActionSets.toArray(new NakedObjectAction[serviceActionSets.size()]);
    }


    public NakedObjectField[] getStaticallyVisibleFields() {
        return getFields();
    }

    protected NakedObjectAction[] getActions(final NakedObjectAction[] availableActions, final Type type) {
        return new NakedObjectAction[0];
    }

    public NakedObject getAggregate(NakedObject object) {
        throw new UnexpectedCallException();
    }

    public NakedObjectAction getClassAction(Type type, String name, NakedObjectSpecification[] parameters) {
        throw new UnexpectedCallException();
    }

    // TODO rename this method - this is for get service actions relating to this type
    public NakedObjectAction[] getClassActions(final NakedObjectAction.Type type) {
        Object[] services = NakedObjectsContext.getObjectLoader().getServices();
        List relatedActions = new ArrayList();
        for (int i = 0; i < services.length; i++) {
            NakedObject serviceAdapter = NakedObjectsContext.getObjectLoader().getAdapterFor(services[i]);
            NakedObjectAction[] serviceActions = serviceAdapter.getSpecification().getObjectActions(type);
            List matchingActions = new ArrayList();
            for (int j = 0; j < serviceActions.length; j++) {
                NakedObjectSpecification returnType = serviceActions[j].getReturnType();
                if (returnType instanceof CollectionSpecification) {
                    // match up with collections of expected type as well as single instances
                    NakedObjectSpecification elementType = ((CollectionSpecification) returnType).getElementType();
                    if (elementType.isOfType(this)) {
                        matchingActions.add(serviceActions[j]);
                    }
                } else if (returnType != null && returnType.isOfType(this)) {
                    matchingActions.add(serviceActions[j]);
                }
            }
            if (matchingActions.size() > 0) {
                NakedObjectAction[] asArray = (NakedObjectAction[]) matchingActions.toArray(new NakedObjectAction[matchingActions
                        .size()]);
                ActionSet set = new ActionSet("id", serviceAdapter.titleString(), asArray);
                relatedActions.add(set);
            }
        }
        return (NakedObjectAction[]) relatedActions.toArray(new NakedObjectAction[relatedActions.size()]);
    }

    public Object getDefaultValue() {
        return null;
    }

    public Object getExtension(Class cls) {
        return extensionsByClass.get(cls);
    }


    public Class[] getExtensions() {
        Class[] classes = new Class[extensionsByClass.size()];
        Object[] objects = extensionsByClass.keySet().toArray();
        System.arraycopy(objects, 0, classes, 0, objects.length);
        return classes;
    }

    /**
     * Represents a set of flags; use {@link Features} to decode.
     * 
     * <p>
     * This is unrelated to the {@link NakedObjectFeature} interface.
     */
    public int getFeatures() {
        return 0;
    }

    // TODO: think this should live on NakedObjectMember - or even NakedObjectFeature - instead.
    public Object getFieldExtension(String name, Class cls) {
        return null;
    }

    // TODO: think this should live on NakedObjectMember - or even NakedObjectFeature - instead.
    public Class[] getFieldExtensions(String name) {
        return new Class[0];
    }

    public NakedObjectField[] getFields() {
        return fields;
    }

    public NakedObjectField[] getFields(Filter filter) {
        NakedObjectField[] allFields = getFields();

        NakedObjectField[] selectedFields = new NakedObjectField[allFields.length];
        int v = 0;
        for (int i = 0; i < allFields.length; i++) {
            if (filter.accept(allFields[i])) {
                selectedFields[v++] = allFields[i];
            }
        }

        NakedObjectField[] fields = new NakedObjectField[v];
        System.arraycopy(selectedFields, 0, fields, 0, v);
        return fields;
    }

    public String getFullName() {
        return className;
    }


    public String getIconName(NakedReference object) {
        return null;
    }

    public NakedObjectAction getObjectAction(Type type, String name, NakedObjectSpecification[] parameters) {
        return null;
    }

    // TODO see getClassActions - do we need to rename this one as well?
    public NakedObjectAction[] getObjectActions(final NakedObjectAction.Type type) {
        NakedObjectAction[] serviceActions;
        if (!Features.isService(this)) {
            serviceActions = findServiceMethodsWithParameter(type);
        } else {
            serviceActions = new NakedObjectAction[0];
        }
        NakedObjectAction[] instanceActions = getActions(objectActions, type);
        NakedObjectAction[] combinedActions = new NakedObjectAction[serviceActions.length + instanceActions.length];
        System.arraycopy(instanceActions, 0, combinedActions, 0, instanceActions.length);
        System.arraycopy(serviceActions, 0, combinedActions, instanceActions.length, serviceActions.length);
        return combinedActions;
    }

    public Class getSearchViaRepository() {
        return null;
    }

    public int getType() {
        return OBJECT;
    }

    public NakedObjectField[] getDynamicallyVisibleFields(NakedObject object) {
        return getFields();
    }

    public boolean hasSubclasses() {
        return false;
    }

    public NakedObjectSpecification[] interfaces() {
        return new NakedObjectSpecification[0];
    }

    /**
     * Discovers what attributes and behaviour the type specified by this specification. As specification are
     * cyclic (specifically a class will reference its subclasses, which in turn reference their superclass)
     * they need be created first, and then later work out its internals. This allows for cyclic references to
     * the be accommodated as there should always a specification available even though it might not be
     * complete.
     */
    public abstract void introspect(ReflectionPeerBuilder builder);


    public boolean isDirty(NakedObject object) {
        return false;
    }

    public boolean isOfType(NakedObjectSpecification specification) {
        return specification == this;
    }

    public Consent isPersistable(NakedObject transientObject) {
        return Allow.DEFAULT;
    }

    public void lifecycleEvent(NakedObject object, int event) {}

    public void markAsService() {}

    public void markDirty(NakedObject object) {}

    public Persistable persistable() {
        return Persistable.USER_PERSISTABLE;
    }

    public boolean queryByExample() {
        return false;
    }

    public NakedObjectSpecification[] subclasses() {
        return new NakedObjectSpecification[0];
    }

    public NakedObjectSpecification superclass() {
        return superClassSpecification;
    }

    public String toString() {
        ToString str = new ToString(this);
        str.append("class", className);
        return str.toString();
    }

}

// Copyright (c) Naked Objects Group Ltd.
