package org.nakedobjects.nof.reflect.spec;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.util.DebugString;
import org.nakedobjects.nof.reflect.peer.ActionPeer;
import org.nakedobjects.nof.reflect.peer.OneToManyPeer;
import org.nakedobjects.nof.reflect.peer.OneToOnePeer;
import org.nakedobjects.nof.reflect.peer.ReflectionException;
import org.nakedobjects.nof.reflect.peer.ReflectionPeerFactory;
import org.nakedobjects.nof.reflect.peer.ValuePeer;
import org.nakedobjects.object.RequiresSetup;


public class ReflectionPeerBuilder implements RequiresSetup {
    private static final Logger LOG = Logger.getLogger(ReflectionPeerBuilder.class);
    private ReflectionPeerFactory[] factories;

    public NakedObjectAction createAction(final ActionPeer actionPeer) {
        ActionPeer peer = actionPeer;
        for (int i = factories.length - 1; i >= 0; i--) {
            peer = factories[i].createAction(peer);
        }
        String actionId = peer.getIdentifier().getName();
        return new ActionImpl(actionId, peer);
    }

    public NakedObjectField createField(final OneToManyPeer fieldPeer) {
        OneToManyPeer peer = fieldPeer;
        for (int i = factories.length - 1; i >= 0; i--) {
            peer = factories[i].createField(peer);
        }
        String fieldId = peer.getIdentifier().getName();
        NakedObjectSpecification specification = peer.getSpecification();
        if (specification == null) {
            String className = fieldPeer.getIdentifier().getClassName();
            throw new ReflectionException("field type for " + className + "." + fieldId + "' must exist");
        }
        return new OneToManyAssociationImpl(fieldId, specification, peer);
    }

    public NakedObjectField createField(final OneToOnePeer fieldPeer) {
        OneToOnePeer peer = fieldPeer;
        for (int i = factories.length - 1; i >= 0; i--) {
            peer = factories[i].createField(peer);
        }
        String fieldId = peer.getIdentifier().getName();
        return new OneToOneAssociationImpl(fieldId, peer.getSpecification(), peer);
    }

    public NakedObjectField createField(final ValuePeer fieldPeer) {
        ValuePeer peer = fieldPeer;
        for (int i = factories.length - 1; i >= 0; i--) {
            peer = factories[i].createField(peer);
        }
        String fieldId = peer.getIdentifier().getName();
        return new ValueAssociationImpl(fieldId, peer.getSpecification(), peer);
    }

    /**
     * Expose as a .Net property.
     * 
     * @property
     */
    public void set_Factories(final ReflectionPeerFactory[] factories) {
        setFactories(factories);
    }

    public void setFactories(final ReflectionPeerFactory[] factories) {
        this.factories = factories;
        LOG.debug("reflection peers:-");
        int len = factories.length;
        if (len == 0) {
            LOG.debug("  none");
        } else {
            for (int i = 0; i < len; i++) {
                LOG.debug("  " + factories[i]);
            }
        }
    }

    public void debugData(DebugString str) {
        str.appendTitle("Reflective peers");
        for (int i = 0; i < factories.length; i++) {
            str.appendln(i + ".", factories[i]);
        }
        str.appendln();
    }

    public void init() {
        for (int i = factories.length - 1; i >= 0; i--) {
            factories[i].init();
        }
    }

    public void shutdown() {
        for (int i = factories.length - 1; i >= 0; i--) {
            factories[i].shutdown();
        }
    }

}
// Copyright (c) Naked Objects Group Ltd.
