package org.nakedobjects.nof.reflect.transaction;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.persist.NakedObjectPersistor;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.reflect.peer.AbstractOneToOnePeer;
import org.nakedobjects.nof.reflect.peer.OneToOnePeer;


public class OneToOneTransaction extends AbstractOneToOnePeer {
    private final static Logger LOG = Logger.getLogger(OneToOneTransaction.class);

    public OneToOneTransaction(final OneToOnePeer local) {
        super(local);
    }

    public void clearAssociation(final NakedObject inObject, final NakedObject associate) {
        NakedObjectPersistor objectManager = NakedObjectsContext.getObjectPersistor();
        if (inObject.getResolveState().isPersistent()) {
            try {
                objectManager.startTransaction();
                super.clearAssociation(inObject, associate);
                objectManager.saveChanges();
                objectManager.endTransaction();
            } catch (RuntimeException e) {
                abort(objectManager);
                throw e;
            }
        } else {
            super.clearAssociation(inObject, associate);
        }
    }

    public void setAssociation(final NakedObject inObject, final NakedObject associate) {
        NakedObjectPersistor objectManager = NakedObjectsContext.getObjectPersistor();
        if (inObject.getResolveState().isPersistent()) {
            try {
                objectManager.startTransaction();
                super.setAssociation(inObject, associate);
                objectManager.saveChanges();
                objectManager.endTransaction();
            } catch (RuntimeException e) {
                abort(objectManager);
                throw e;
            }
        } else {
            super.setAssociation(inObject, associate);
        }
    }

    private void abort(final NakedObjectPersistor objectManager) {
        LOG.info("exception executing " + getIdentifier() + ", aborting transaction");
        try {
            objectManager.abortTransaction();
        } catch (Exception e2) {
            LOG.error("failure during abort", e2);
        }
    }

}
// Copyright (c) Naked Objects Group Ltd.
