package org.nakedobjects.nof.reflect.transaction;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedValue;
import org.nakedobjects.noa.persist.NakedObjectPersistor;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.reflect.peer.AbstractValuePeer;
import org.nakedobjects.nof.reflect.peer.ValuePeer;


public class ValueTransaction extends AbstractValuePeer {
    private final static Logger LOG = Logger.getLogger(ValueTransaction.class);

    public ValueTransaction(final ValuePeer local) {
        super(local);
    }

    public void clearValue(final NakedObject inObject) {
        NakedObjectPersistor objectManager = NakedObjectsContext.getObjectPersistor();
        if (inObject.getResolveState().isPersistent()) {
            try {
                objectManager.startTransaction();
                super.clearValue(inObject);
                objectManager.saveChanges();
                objectManager.endTransaction();
            } catch (RuntimeException e) {
                abort(objectManager);
                throw e;
            }
        } else {
            super.clearValue(inObject);
        }

    }

    public void setValue(final NakedObject inObject, final NakedValue value) {
        NakedObjectPersistor objectManager = NakedObjectsContext.getObjectPersistor();
        if (inObject.getResolveState().isPersistent()) {
            try {
                objectManager.startTransaction();
                super.setValue(inObject, value);
                objectManager.saveChanges();
                objectManager.endTransaction();
            } catch (RuntimeException e) {
                abort(objectManager);
                throw e;
            }
        } else {
            super.setValue(inObject, value);
        }

    }

    private void abort(final NakedObjectPersistor objectManager) {
        LOG.info("exception executing " + getIdentifier() + ", aborting transaction");
        try {
            objectManager.abortTransaction();
        } catch (Exception e2) {
            LOG.error("failure during abort", e2);
        }
    }

}
// Copyright (c) Naked Objects Group Ltd.
